// ============================================================================
// COPYRIGHT NOTICE
// ----------------------------------------------------------------------------
// (This is the open source ISC license, see
// http://en.wikipedia.org/wiki/ISC_license
// for more info)
//
// Copyright © 2012-2023  Andreas M. Rammelt <rammi@caff.de>
//
// Permission to use, copy, modify, and/or distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//=============================================================================
// Latest version on https://caff.de/projects/decaff-commons/
//=============================================================================
package de.caff.generics.mda;

import de.caff.annotation.NotNull;

/**
 * Basic interface for accessing indexable multi-dimensional items.
 * @author <a href="mailto:rammi@caff.de">Rammi</a>
 * @since November 02, 2019
 */
public interface MultiDimensional
{
  /**
   * Get the number of dimensions used by this access,
   * @return number of dimensions
   */
  int getNumDimensions();

  /**
   * Get the size of the given dimension.
   * @param dim dimension, greater than {@code 0} and less than {@link #getNumDimensions()}
   * @return number of elements in the given dimension
   */
  int getSize(int dim);

  /**
   * Get the sizes of the dimensions of this access.
   * @return array of length {@link #getNumDimensions()}
   */
  @NotNull
  int[] getSizes();

  /**
   * Get the combined number of elements in all dimensions.
   * @return number of all elements in this multi-dimensional array
   */
  long getNumElements();
}
