// ============================================================================
// COPYRIGHT NOTICE
// ----------------------------------------------------------------------------
// (This is the open source ISC license, see
// http://en.wikipedia.org/wiki/ISC_license
// for more info)
//
// Copyright © 2021 - 2022  Andreas M. Rammelt <rammi@caff.de>
//
// Permission to use, copy, modify, and/or distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//=============================================================================
// Latest version on https://caff.de/projects/decaff-commons/
//=============================================================================
package de.caff.generics.tuple;

import de.caff.annotation.NotNull;
import de.caff.annotation.Nullable;

import java.io.Serializable;
import java.util.Objects;

/**
 * Immutable tuple with 5 nullable elements.
 *
 * If you want to iterate over a tuple or access its elements by index
 * (note that both is only possible for a common super type of the tuple's types)
 * {@link de.caff.generics.Indexable#viewTuple(ITuple5)} will come to help.
 *
 * Note that this class is automatically created by {@code tools.TupleClassCreator} (not yet public).
 *
 * @param <T1> type of first element
 * @param <T2> type of second element
 * @param <T3> type of third element
 * @param <T4> type of fourth element
 * @param <T5> type of fifth element
 * @author <a href="mailto:rammi@caff.de">Rammi</a>
 * @since November 17, 2021
 * @see Tuple5
 */
public class NTuple5<T1, T2, T3, T4, T5>
        extends ITuple5.Base<T1, T2, T3, T4, T5>
        implements Serializable
{
  private static final long serialVersionUID = 1L;

  @Nullable
  private final T1 elem1;
  @Nullable
  private final T2 elem2;
  @Nullable
  private final T3 elem3;
  @Nullable
  private final T4 elem4;
  @Nullable
  private final T5 elem5;

  /**
   * Constructor.
   * @param e1 first element
   * @param e2 second element
   * @param e3 third element
   * @param e4 fourth element
   * @param e5 fifth element
   */
  public NTuple5(@Nullable T1 e1, @Nullable T2 e2, @Nullable T3 e3, @Nullable T4 e4, @Nullable T5 e5)
  {
    elem1 = e1;
    elem2 = e2;
    elem3 = e3;
    elem4 = e4;
    elem5 = e5;
  }

  @Nullable
  @Override
  public T1 _1()  {
    return elem1;
  }

  @Nullable
  @Override
  public T2 _2()  {
    return elem2;
  }

  @Nullable
  @Override
  public T3 _3()  {
    return elem3;
  }

  @Nullable
  @Override
  public T4 _4()  {
    return elem4;
  }

  @Nullable
  @Override
  public T5 _5()  {
    return elem5;
  }

  @NotNull
  @Override
  public NTuple5<T1, T2, T3, T5, T4> _12354()  {
    return new NTuple5<>(elem1, elem2, elem3, elem5, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T1, T2, T5, T3, T4> _12534()  {
    return new NTuple5<>(elem1, elem2, elem5, elem3, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T1, T5, T2, T3, T4> _15234()  {
    return new NTuple5<>(elem1, elem5, elem2, elem3, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T5, T1, T2, T3, T4> _51234()  {
    return new NTuple5<>(elem5, elem1, elem2, elem3, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T5, T1, T2, T4, T3> _51243()  {
    return new NTuple5<>(elem5, elem1, elem2, elem4, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T1, T5, T2, T4, T3> _15243()  {
    return new NTuple5<>(elem1, elem5, elem2, elem4, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T1, T2, T5, T4, T3> _12543()  {
    return new NTuple5<>(elem1, elem2, elem5, elem4, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T1, T2, T4, T5, T3> _12453()  {
    return new NTuple5<>(elem1, elem2, elem4, elem5, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T1, T2, T4, T3, T5> _12435()  {
    return new NTuple5<>(elem1, elem2, elem4, elem3, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T1, T4, T2, T3, T5> _14235()  {
    return new NTuple5<>(elem1, elem4, elem2, elem3, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T1, T4, T2, T5, T3> _14253()  {
    return new NTuple5<>(elem1, elem4, elem2, elem5, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T1, T4, T5, T2, T3> _14523()  {
    return new NTuple5<>(elem1, elem4, elem5, elem2, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T1, T5, T4, T2, T3> _15423()  {
    return new NTuple5<>(elem1, elem5, elem4, elem2, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T5, T1, T4, T2, T3> _51423()  {
    return new NTuple5<>(elem5, elem1, elem4, elem2, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T5, T4, T1, T2, T3> _54123()  {
    return new NTuple5<>(elem5, elem4, elem1, elem2, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T4, T5, T1, T2, T3> _45123()  {
    return new NTuple5<>(elem4, elem5, elem1, elem2, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T4, T1, T5, T2, T3> _41523()  {
    return new NTuple5<>(elem4, elem1, elem5, elem2, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T4, T1, T2, T5, T3> _41253()  {
    return new NTuple5<>(elem4, elem1, elem2, elem5, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T4, T1, T2, T3, T5> _41235()  {
    return new NTuple5<>(elem4, elem1, elem2, elem3, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T4, T1, T3, T2, T5> _41325()  {
    return new NTuple5<>(elem4, elem1, elem3, elem2, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T4, T1, T3, T5, T2> _41352()  {
    return new NTuple5<>(elem4, elem1, elem3, elem5, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T4, T1, T5, T3, T2> _41532()  {
    return new NTuple5<>(elem4, elem1, elem5, elem3, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T4, T5, T1, T3, T2> _45132()  {
    return new NTuple5<>(elem4, elem5, elem1, elem3, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T5, T4, T1, T3, T2> _54132()  {
    return new NTuple5<>(elem5, elem4, elem1, elem3, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T5, T1, T4, T3, T2> _51432()  {
    return new NTuple5<>(elem5, elem1, elem4, elem3, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T1, T5, T4, T3, T2> _15432()  {
    return new NTuple5<>(elem1, elem5, elem4, elem3, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T1, T4, T5, T3, T2> _14532()  {
    return new NTuple5<>(elem1, elem4, elem5, elem3, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T1, T4, T3, T5, T2> _14352()  {
    return new NTuple5<>(elem1, elem4, elem3, elem5, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T1, T4, T3, T2, T5> _14325()  {
    return new NTuple5<>(elem1, elem4, elem3, elem2, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T1, T3, T4, T2, T5> _13425()  {
    return new NTuple5<>(elem1, elem3, elem4, elem2, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T1, T3, T4, T5, T2> _13452()  {
    return new NTuple5<>(elem1, elem3, elem4, elem5, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T1, T3, T5, T4, T2> _13542()  {
    return new NTuple5<>(elem1, elem3, elem5, elem4, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T1, T5, T3, T4, T2> _15342()  {
    return new NTuple5<>(elem1, elem5, elem3, elem4, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T5, T1, T3, T4, T2> _51342()  {
    return new NTuple5<>(elem5, elem1, elem3, elem4, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T5, T1, T3, T2, T4> _51324()  {
    return new NTuple5<>(elem5, elem1, elem3, elem2, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T1, T5, T3, T2, T4> _15324()  {
    return new NTuple5<>(elem1, elem5, elem3, elem2, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T1, T3, T5, T2, T4> _13524()  {
    return new NTuple5<>(elem1, elem3, elem5, elem2, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T1, T3, T2, T5, T4> _13254()  {
    return new NTuple5<>(elem1, elem3, elem2, elem5, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T1, T3, T2, T4, T5> _13245()  {
    return new NTuple5<>(elem1, elem3, elem2, elem4, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T3, T1, T2, T4, T5> _31245()  {
    return new NTuple5<>(elem3, elem1, elem2, elem4, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T3, T1, T2, T5, T4> _31254()  {
    return new NTuple5<>(elem3, elem1, elem2, elem5, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T3, T1, T5, T2, T4> _31524()  {
    return new NTuple5<>(elem3, elem1, elem5, elem2, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T3, T5, T1, T2, T4> _35124()  {
    return new NTuple5<>(elem3, elem5, elem1, elem2, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T5, T3, T1, T2, T4> _53124()  {
    return new NTuple5<>(elem5, elem3, elem1, elem2, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T5, T3, T1, T4, T2> _53142()  {
    return new NTuple5<>(elem5, elem3, elem1, elem4, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T3, T5, T1, T4, T2> _35142()  {
    return new NTuple5<>(elem3, elem5, elem1, elem4, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T3, T1, T5, T4, T2> _31542()  {
    return new NTuple5<>(elem3, elem1, elem5, elem4, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T3, T1, T4, T5, T2> _31452()  {
    return new NTuple5<>(elem3, elem1, elem4, elem5, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T3, T1, T4, T2, T5> _31425()  {
    return new NTuple5<>(elem3, elem1, elem4, elem2, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T3, T4, T1, T2, T5> _34125()  {
    return new NTuple5<>(elem3, elem4, elem1, elem2, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T3, T4, T1, T5, T2> _34152()  {
    return new NTuple5<>(elem3, elem4, elem1, elem5, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T3, T4, T5, T1, T2> _34512()  {
    return new NTuple5<>(elem3, elem4, elem5, elem1, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T3, T5, T4, T1, T2> _35412()  {
    return new NTuple5<>(elem3, elem5, elem4, elem1, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T5, T3, T4, T1, T2> _53412()  {
    return new NTuple5<>(elem5, elem3, elem4, elem1, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T5, T4, T3, T1, T2> _54312()  {
    return new NTuple5<>(elem5, elem4, elem3, elem1, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T4, T5, T3, T1, T2> _45312()  {
    return new NTuple5<>(elem4, elem5, elem3, elem1, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T4, T3, T5, T1, T2> _43512()  {
    return new NTuple5<>(elem4, elem3, elem5, elem1, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T4, T3, T1, T5, T2> _43152()  {
    return new NTuple5<>(elem4, elem3, elem1, elem5, elem2);
  }

  @NotNull
  @Override
  public NTuple5<T4, T3, T1, T2, T5> _43125()  {
    return new NTuple5<>(elem4, elem3, elem1, elem2, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T4, T3, T2, T1, T5> _43215()  {
    return new NTuple5<>(elem4, elem3, elem2, elem1, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T4, T3, T2, T5, T1> _43251()  {
    return new NTuple5<>(elem4, elem3, elem2, elem5, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T4, T3, T5, T2, T1> _43521()  {
    return new NTuple5<>(elem4, elem3, elem5, elem2, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T4, T5, T3, T2, T1> _45321()  {
    return new NTuple5<>(elem4, elem5, elem3, elem2, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T5, T4, T3, T2, T1> _54321()  {
    return new NTuple5<>(elem5, elem4, elem3, elem2, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T5, T3, T4, T2, T1> _53421()  {
    return new NTuple5<>(elem5, elem3, elem4, elem2, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T3, T5, T4, T2, T1> _35421()  {
    return new NTuple5<>(elem3, elem5, elem4, elem2, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T3, T4, T5, T2, T1> _34521()  {
    return new NTuple5<>(elem3, elem4, elem5, elem2, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T3, T4, T2, T5, T1> _34251()  {
    return new NTuple5<>(elem3, elem4, elem2, elem5, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T3, T4, T2, T1, T5> _34215()  {
    return new NTuple5<>(elem3, elem4, elem2, elem1, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T3, T2, T4, T1, T5> _32415()  {
    return new NTuple5<>(elem3, elem2, elem4, elem1, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T3, T2, T4, T5, T1> _32451()  {
    return new NTuple5<>(elem3, elem2, elem4, elem5, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T3, T2, T5, T4, T1> _32541()  {
    return new NTuple5<>(elem3, elem2, elem5, elem4, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T3, T5, T2, T4, T1> _35241()  {
    return new NTuple5<>(elem3, elem5, elem2, elem4, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T5, T3, T2, T4, T1> _53241()  {
    return new NTuple5<>(elem5, elem3, elem2, elem4, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T5, T3, T2, T1, T4> _53214()  {
    return new NTuple5<>(elem5, elem3, elem2, elem1, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T3, T5, T2, T1, T4> _35214()  {
    return new NTuple5<>(elem3, elem5, elem2, elem1, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T3, T2, T5, T1, T4> _32514()  {
    return new NTuple5<>(elem3, elem2, elem5, elem1, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T3, T2, T1, T5, T4> _32154()  {
    return new NTuple5<>(elem3, elem2, elem1, elem5, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T3, T2, T1, T4, T5> _32145()  {
    return new NTuple5<>(elem3, elem2, elem1, elem4, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T2, T3, T1, T4, T5> _23145()  {
    return new NTuple5<>(elem2, elem3, elem1, elem4, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T2, T3, T1, T5, T4> _23154()  {
    return new NTuple5<>(elem2, elem3, elem1, elem5, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T2, T3, T5, T1, T4> _23514()  {
    return new NTuple5<>(elem2, elem3, elem5, elem1, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T2, T5, T3, T1, T4> _25314()  {
    return new NTuple5<>(elem2, elem5, elem3, elem1, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T5, T2, T3, T1, T4> _52314()  {
    return new NTuple5<>(elem5, elem2, elem3, elem1, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T5, T2, T3, T4, T1> _52341()  {
    return new NTuple5<>(elem5, elem2, elem3, elem4, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T2, T5, T3, T4, T1> _25341()  {
    return new NTuple5<>(elem2, elem5, elem3, elem4, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T2, T3, T5, T4, T1> _23541()  {
    return new NTuple5<>(elem2, elem3, elem5, elem4, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T2, T3, T4, T5, T1> _23451()  {
    return new NTuple5<>(elem2, elem3, elem4, elem5, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T2, T3, T4, T1, T5> _23415()  {
    return new NTuple5<>(elem2, elem3, elem4, elem1, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T2, T4, T3, T1, T5> _24315()  {
    return new NTuple5<>(elem2, elem4, elem3, elem1, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T2, T4, T3, T5, T1> _24351()  {
    return new NTuple5<>(elem2, elem4, elem3, elem5, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T2, T4, T5, T3, T1> _24531()  {
    return new NTuple5<>(elem2, elem4, elem5, elem3, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T2, T5, T4, T3, T1> _25431()  {
    return new NTuple5<>(elem2, elem5, elem4, elem3, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T5, T2, T4, T3, T1> _52431()  {
    return new NTuple5<>(elem5, elem2, elem4, elem3, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T5, T4, T2, T3, T1> _54231()  {
    return new NTuple5<>(elem5, elem4, elem2, elem3, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T4, T5, T2, T3, T1> _45231()  {
    return new NTuple5<>(elem4, elem5, elem2, elem3, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T4, T2, T5, T3, T1> _42531()  {
    return new NTuple5<>(elem4, elem2, elem5, elem3, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T4, T2, T3, T5, T1> _42351()  {
    return new NTuple5<>(elem4, elem2, elem3, elem5, elem1);
  }

  @NotNull
  @Override
  public NTuple5<T4, T2, T3, T1, T5> _42315()  {
    return new NTuple5<>(elem4, elem2, elem3, elem1, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T4, T2, T1, T3, T5> _42135()  {
    return new NTuple5<>(elem4, elem2, elem1, elem3, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T4, T2, T1, T5, T3> _42153()  {
    return new NTuple5<>(elem4, elem2, elem1, elem5, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T4, T2, T5, T1, T3> _42513()  {
    return new NTuple5<>(elem4, elem2, elem5, elem1, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T4, T5, T2, T1, T3> _45213()  {
    return new NTuple5<>(elem4, elem5, elem2, elem1, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T5, T4, T2, T1, T3> _54213()  {
    return new NTuple5<>(elem5, elem4, elem2, elem1, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T5, T2, T4, T1, T3> _52413()  {
    return new NTuple5<>(elem5, elem2, elem4, elem1, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T2, T5, T4, T1, T3> _25413()  {
    return new NTuple5<>(elem2, elem5, elem4, elem1, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T2, T4, T5, T1, T3> _24513()  {
    return new NTuple5<>(elem2, elem4, elem5, elem1, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T2, T4, T1, T5, T3> _24153()  {
    return new NTuple5<>(elem2, elem4, elem1, elem5, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T2, T4, T1, T3, T5> _24135()  {
    return new NTuple5<>(elem2, elem4, elem1, elem3, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T2, T1, T4, T3, T5> _21435()  {
    return new NTuple5<>(elem2, elem1, elem4, elem3, elem5);
  }

  @NotNull
  @Override
  public NTuple5<T2, T1, T4, T5, T3> _21453()  {
    return new NTuple5<>(elem2, elem1, elem4, elem5, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T2, T1, T5, T4, T3> _21543()  {
    return new NTuple5<>(elem2, elem1, elem5, elem4, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T2, T5, T1, T4, T3> _25143()  {
    return new NTuple5<>(elem2, elem5, elem1, elem4, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T5, T2, T1, T4, T3> _52143()  {
    return new NTuple5<>(elem5, elem2, elem1, elem4, elem3);
  }

  @NotNull
  @Override
  public NTuple5<T5, T2, T1, T3, T4> _52134()  {
    return new NTuple5<>(elem5, elem2, elem1, elem3, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T2, T5, T1, T3, T4> _25134()  {
    return new NTuple5<>(elem2, elem5, elem1, elem3, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T2, T1, T5, T3, T4> _21534()  {
    return new NTuple5<>(elem2, elem1, elem5, elem3, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T2, T1, T3, T5, T4> _21354()  {
    return new NTuple5<>(elem2, elem1, elem3, elem5, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T2, T1, T3, T4, T5> _21345()  {
    return new NTuple5<>(elem2, elem1, elem3, elem4, elem5);
  }

  @NotNull
  @Override
  public NTuple4<T1, T2, T3, T4> _1234()  {
    return new NTuple4<>(elem1, elem2, elem3, elem4);
  }

  @NotNull
  @Override
  public NTuple4<T1, T2, T3, T5> _1235()  {
    return new NTuple4<>(elem1, elem2, elem3, elem5);
  }

  @NotNull
  @Override
  public NTuple4<T1, T2, T4, T5> _1245()  {
    return new NTuple4<>(elem1, elem2, elem4, elem5);
  }

  @NotNull
  @Override
  public NTuple4<T1, T3, T4, T5> _1345()  {
    return new NTuple4<>(elem1, elem3, elem4, elem5);
  }

  @NotNull
  @Override
  public NTuple4<T2, T3, T4, T5> _2345()  {
    return new NTuple4<>(elem2, elem3, elem4, elem5);
  }

  @NotNull
  @Override
  public NTuple3<T1, T2, T3> _123()  {
    return new NTuple3<>(elem1, elem2, elem3);
  }

  @NotNull
  @Override
  public NTuple3<T1, T3, T2> _132()  {
    return new NTuple3<>(elem1, elem3, elem2);
  }

  @NotNull
  @Override
  public NTuple3<T3, T1, T2> _312()  {
    return new NTuple3<>(elem3, elem1, elem2);
  }

  @NotNull
  @Override
  public NTuple3<T3, T2, T1> _321()  {
    return new NTuple3<>(elem3, elem2, elem1);
  }

  @NotNull
  @Override
  public NTuple3<T2, T3, T1> _231()  {
    return new NTuple3<>(elem2, elem3, elem1);
  }

  @NotNull
  @Override
  public NTuple3<T2, T1, T3> _213()  {
    return new NTuple3<>(elem2, elem1, elem3);
  }

  @NotNull
  @Override
  public NTuple3<T1, T2, T4> _124()  {
    return new NTuple3<>(elem1, elem2, elem4);
  }

  @NotNull
  @Override
  public NTuple3<T1, T4, T2> _142()  {
    return new NTuple3<>(elem1, elem4, elem2);
  }

  @NotNull
  @Override
  public NTuple3<T4, T1, T2> _412()  {
    return new NTuple3<>(elem4, elem1, elem2);
  }

  @NotNull
  @Override
  public NTuple3<T4, T2, T1> _421()  {
    return new NTuple3<>(elem4, elem2, elem1);
  }

  @NotNull
  @Override
  public NTuple3<T2, T4, T1> _241()  {
    return new NTuple3<>(elem2, elem4, elem1);
  }

  @NotNull
  @Override
  public NTuple3<T2, T1, T4> _214()  {
    return new NTuple3<>(elem2, elem1, elem4);
  }

  @NotNull
  @Override
  public NTuple3<T1, T3, T4> _134()  {
    return new NTuple3<>(elem1, elem3, elem4);
  }

  @NotNull
  @Override
  public NTuple3<T1, T4, T3> _143()  {
    return new NTuple3<>(elem1, elem4, elem3);
  }

  @NotNull
  @Override
  public NTuple3<T4, T1, T3> _413()  {
    return new NTuple3<>(elem4, elem1, elem3);
  }

  @NotNull
  @Override
  public NTuple3<T4, T3, T1> _431()  {
    return new NTuple3<>(elem4, elem3, elem1);
  }

  @NotNull
  @Override
  public NTuple3<T3, T4, T1> _341()  {
    return new NTuple3<>(elem3, elem4, elem1);
  }

  @NotNull
  @Override
  public NTuple3<T3, T1, T4> _314()  {
    return new NTuple3<>(elem3, elem1, elem4);
  }

  @NotNull
  @Override
  public NTuple3<T2, T3, T4> _234()  {
    return new NTuple3<>(elem2, elem3, elem4);
  }

  @NotNull
  @Override
  public NTuple3<T2, T4, T3> _243()  {
    return new NTuple3<>(elem2, elem4, elem3);
  }

  @NotNull
  @Override
  public NTuple3<T4, T2, T3> _423()  {
    return new NTuple3<>(elem4, elem2, elem3);
  }

  @NotNull
  @Override
  public NTuple3<T4, T3, T2> _432()  {
    return new NTuple3<>(elem4, elem3, elem2);
  }

  @NotNull
  @Override
  public NTuple3<T3, T4, T2> _342()  {
    return new NTuple3<>(elem3, elem4, elem2);
  }

  @NotNull
  @Override
  public NTuple3<T3, T2, T4> _324()  {
    return new NTuple3<>(elem3, elem2, elem4);
  }

  @NotNull
  @Override
  public NTuple3<T1, T2, T5> _125()  {
    return new NTuple3<>(elem1, elem2, elem5);
  }

  @NotNull
  @Override
  public NTuple3<T1, T5, T2> _152()  {
    return new NTuple3<>(elem1, elem5, elem2);
  }

  @NotNull
  @Override
  public NTuple3<T5, T1, T2> _512()  {
    return new NTuple3<>(elem5, elem1, elem2);
  }

  @NotNull
  @Override
  public NTuple3<T5, T2, T1> _521()  {
    return new NTuple3<>(elem5, elem2, elem1);
  }

  @NotNull
  @Override
  public NTuple3<T2, T5, T1> _251()  {
    return new NTuple3<>(elem2, elem5, elem1);
  }

  @NotNull
  @Override
  public NTuple3<T2, T1, T5> _215()  {
    return new NTuple3<>(elem2, elem1, elem5);
  }

  @NotNull
  @Override
  public NTuple3<T1, T3, T5> _135()  {
    return new NTuple3<>(elem1, elem3, elem5);
  }

  @NotNull
  @Override
  public NTuple3<T1, T5, T3> _153()  {
    return new NTuple3<>(elem1, elem5, elem3);
  }

  @NotNull
  @Override
  public NTuple3<T5, T1, T3> _513()  {
    return new NTuple3<>(elem5, elem1, elem3);
  }

  @NotNull
  @Override
  public NTuple3<T5, T3, T1> _531()  {
    return new NTuple3<>(elem5, elem3, elem1);
  }

  @NotNull
  @Override
  public NTuple3<T3, T5, T1> _351()  {
    return new NTuple3<>(elem3, elem5, elem1);
  }

  @NotNull
  @Override
  public NTuple3<T3, T1, T5> _315()  {
    return new NTuple3<>(elem3, elem1, elem5);
  }

  @NotNull
  @Override
  public NTuple3<T2, T3, T5> _235()  {
    return new NTuple3<>(elem2, elem3, elem5);
  }

  @NotNull
  @Override
  public NTuple3<T2, T5, T3> _253()  {
    return new NTuple3<>(elem2, elem5, elem3);
  }

  @NotNull
  @Override
  public NTuple3<T5, T2, T3> _523()  {
    return new NTuple3<>(elem5, elem2, elem3);
  }

  @NotNull
  @Override
  public NTuple3<T5, T3, T2> _532()  {
    return new NTuple3<>(elem5, elem3, elem2);
  }

  @NotNull
  @Override
  public NTuple3<T3, T5, T2> _352()  {
    return new NTuple3<>(elem3, elem5, elem2);
  }

  @NotNull
  @Override
  public NTuple3<T3, T2, T5> _325()  {
    return new NTuple3<>(elem3, elem2, elem5);
  }

  @NotNull
  @Override
  public NTuple3<T1, T4, T5> _145()  {
    return new NTuple3<>(elem1, elem4, elem5);
  }

  @NotNull
  @Override
  public NTuple3<T1, T5, T4> _154()  {
    return new NTuple3<>(elem1, elem5, elem4);
  }

  @NotNull
  @Override
  public NTuple3<T5, T1, T4> _514()  {
    return new NTuple3<>(elem5, elem1, elem4);
  }

  @NotNull
  @Override
  public NTuple3<T5, T4, T1> _541()  {
    return new NTuple3<>(elem5, elem4, elem1);
  }

  @NotNull
  @Override
  public NTuple3<T4, T5, T1> _451()  {
    return new NTuple3<>(elem4, elem5, elem1);
  }

  @NotNull
  @Override
  public NTuple3<T4, T1, T5> _415()  {
    return new NTuple3<>(elem4, elem1, elem5);
  }

  @NotNull
  @Override
  public NTuple3<T2, T4, T5> _245()  {
    return new NTuple3<>(elem2, elem4, elem5);
  }

  @NotNull
  @Override
  public NTuple3<T2, T5, T4> _254()  {
    return new NTuple3<>(elem2, elem5, elem4);
  }

  @NotNull
  @Override
  public NTuple3<T5, T2, T4> _524()  {
    return new NTuple3<>(elem5, elem2, elem4);
  }

  @NotNull
  @Override
  public NTuple3<T5, T4, T2> _542()  {
    return new NTuple3<>(elem5, elem4, elem2);
  }

  @NotNull
  @Override
  public NTuple3<T4, T5, T2> _452()  {
    return new NTuple3<>(elem4, elem5, elem2);
  }

  @NotNull
  @Override
  public NTuple3<T4, T2, T5> _425()  {
    return new NTuple3<>(elem4, elem2, elem5);
  }

  @NotNull
  @Override
  public NTuple3<T3, T4, T5> _345()  {
    return new NTuple3<>(elem3, elem4, elem5);
  }

  @NotNull
  @Override
  public NTuple3<T3, T5, T4> _354()  {
    return new NTuple3<>(elem3, elem5, elem4);
  }

  @NotNull
  @Override
  public NTuple3<T5, T3, T4> _534()  {
    return new NTuple3<>(elem5, elem3, elem4);
  }

  @NotNull
  @Override
  public NTuple3<T5, T4, T3> _543()  {
    return new NTuple3<>(elem5, elem4, elem3);
  }

  @NotNull
  @Override
  public NTuple3<T4, T5, T3> _453()  {
    return new NTuple3<>(elem4, elem5, elem3);
  }

  @NotNull
  @Override
  public NTuple3<T4, T3, T5> _435()  {
    return new NTuple3<>(elem4, elem3, elem5);
  }

  @NotNull
  @Override
  public NTuple2<T1, T2> _12()  {
    return new NTuple2<>(elem1, elem2);
  }

  @NotNull
  @Override
  public NTuple2<T2, T1> _21()  {
    return new NTuple2<>(elem2, elem1);
  }

  @NotNull
  @Override
  public NTuple2<T1, T3> _13()  {
    return new NTuple2<>(elem1, elem3);
  }

  @NotNull
  @Override
  public NTuple2<T3, T1> _31()  {
    return new NTuple2<>(elem3, elem1);
  }

  @NotNull
  @Override
  public NTuple2<T2, T3> _23()  {
    return new NTuple2<>(elem2, elem3);
  }

  @NotNull
  @Override
  public NTuple2<T3, T2> _32()  {
    return new NTuple2<>(elem3, elem2);
  }

  @NotNull
  @Override
  public NTuple2<T1, T4> _14()  {
    return new NTuple2<>(elem1, elem4);
  }

  @NotNull
  @Override
  public NTuple2<T4, T1> _41()  {
    return new NTuple2<>(elem4, elem1);
  }

  @NotNull
  @Override
  public NTuple2<T2, T4> _24()  {
    return new NTuple2<>(elem2, elem4);
  }

  @NotNull
  @Override
  public NTuple2<T4, T2> _42()  {
    return new NTuple2<>(elem4, elem2);
  }

  @NotNull
  @Override
  public NTuple2<T3, T4> _34()  {
    return new NTuple2<>(elem3, elem4);
  }

  @NotNull
  @Override
  public NTuple2<T4, T3> _43()  {
    return new NTuple2<>(elem4, elem3);
  }

  @NotNull
  @Override
  public NTuple2<T1, T5> _15()  {
    return new NTuple2<>(elem1, elem5);
  }

  @NotNull
  @Override
  public NTuple2<T5, T1> _51()  {
    return new NTuple2<>(elem5, elem1);
  }

  @NotNull
  @Override
  public NTuple2<T2, T5> _25()  {
    return new NTuple2<>(elem2, elem5);
  }

  @NotNull
  @Override
  public NTuple2<T5, T2> _52()  {
    return new NTuple2<>(elem5, elem2);
  }

  @NotNull
  @Override
  public NTuple2<T3, T5> _35()  {
    return new NTuple2<>(elem3, elem5);
  }

  @NotNull
  @Override
  public NTuple2<T5, T3> _53()  {
    return new NTuple2<>(elem5, elem3);
  }

  @NotNull
  @Override
  public NTuple2<T4, T5> _45()  {
    return new NTuple2<>(elem4, elem5);
  }

  @NotNull
  @Override
  public NTuple2<T5, T4> _54()  {
    return new NTuple2<>(elem5, elem4);
  }

  @NotNull
  @Override
  public NTuple5<T1, T2, T3, T4, T5> frozen()
  {
    return this;
  }

  /**
   * Create a 5-tuple from this one where the first element is exchanged. 
   * @param e1 new first element
   * @return tuple with exchanged element
   * @param <E1> type of exchanged first element
   */
  @NotNull
  public <E1> NTuple5<E1, T2, T3, T4, T5> x1(@Nullable E1 e1)  {
    return new NTuple5<>(e1, elem2, elem3, elem4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the second element is exchanged. 
   * @param e2 new second element
   * @return tuple with exchanged element
   * @param <E2> type of exchanged second element
   */
  @NotNull
  public <E2> NTuple5<T1, E2, T3, T4, T5> x2(@Nullable E2 e2)  {
    return new NTuple5<>(elem1, e2, elem3, elem4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the third element is exchanged. 
   * @param e3 new third element
   * @return tuple with exchanged element
   * @param <E3> type of exchanged third element
   */
  @NotNull
  public <E3> NTuple5<T1, T2, E3, T4, T5> x3(@Nullable E3 e3)  {
    return new NTuple5<>(elem1, elem2, e3, elem4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the fourth element is exchanged. 
   * @param e4 new fourth element
   * @return tuple with exchanged element
   * @param <E4> type of exchanged fourth element
   */
  @NotNull
  public <E4> NTuple5<T1, T2, T3, E4, T5> x4(@Nullable E4 e4)  {
    return new NTuple5<>(elem1, elem2, elem3, e4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the fifth element is exchanged. 
   * @param e5 new fifth element
   * @return tuple with exchanged element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E5> NTuple5<T1, T2, T3, T4, E5> x5(@Nullable E5 e5)  {
    return new NTuple5<>(elem1, elem2, elem3, elem4, e5);
  }

  /**
   * Create a 5-tuple from this one where the first and second elements are exchanged. 
   * @param e1 new first element
   * @param e2 new second element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E2> type of exchanged second element
   */
  @NotNull
  public <E1, E2> NTuple5<E1, E2, T3, T4, T5> x12(@Nullable E1 e1, @Nullable E2 e2)  {
    return new NTuple5<>(e1, e2, elem3, elem4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the first and third elements are exchanged. 
   * @param e1 new first element
   * @param e3 new third element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E3> type of exchanged third element
   */
  @NotNull
  public <E1, E3> NTuple5<E1, T2, E3, T4, T5> x13(@Nullable E1 e1, @Nullable E3 e3)  {
    return new NTuple5<>(e1, elem2, e3, elem4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the second and third elements are exchanged. 
   * @param e2 new second element
   * @param e3 new third element
   * @return tuple with exchanged elements
   * @param <E2> type of exchanged second element
   * @param <E3> type of exchanged third element
   */
  @NotNull
  public <E2, E3> NTuple5<T1, E2, E3, T4, T5> x23(@Nullable E2 e2, @Nullable E3 e3)  {
    return new NTuple5<>(elem1, e2, e3, elem4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the first and fourth elements are exchanged. 
   * @param e1 new first element
   * @param e4 new fourth element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E4> type of exchanged fourth element
   */
  @NotNull
  public <E1, E4> NTuple5<E1, T2, T3, E4, T5> x14(@Nullable E1 e1, @Nullable E4 e4)  {
    return new NTuple5<>(e1, elem2, elem3, e4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the second and fourth elements are exchanged. 
   * @param e2 new second element
   * @param e4 new fourth element
   * @return tuple with exchanged elements
   * @param <E2> type of exchanged second element
   * @param <E4> type of exchanged fourth element
   */
  @NotNull
  public <E2, E4> NTuple5<T1, E2, T3, E4, T5> x24(@Nullable E2 e2, @Nullable E4 e4)  {
    return new NTuple5<>(elem1, e2, elem3, e4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the third and fourth elements are exchanged. 
   * @param e3 new third element
   * @param e4 new fourth element
   * @return tuple with exchanged elements
   * @param <E3> type of exchanged third element
   * @param <E4> type of exchanged fourth element
   */
  @NotNull
  public <E3, E4> NTuple5<T1, T2, E3, E4, T5> x34(@Nullable E3 e3, @Nullable E4 e4)  {
    return new NTuple5<>(elem1, elem2, e3, e4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the first and fifth elements are exchanged. 
   * @param e1 new first element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E1, E5> NTuple5<E1, T2, T3, T4, E5> x15(@Nullable E1 e1, @Nullable E5 e5)  {
    return new NTuple5<>(e1, elem2, elem3, elem4, e5);
  }

  /**
   * Create a 5-tuple from this one where the second and fifth elements are exchanged. 
   * @param e2 new second element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E2> type of exchanged second element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E2, E5> NTuple5<T1, E2, T3, T4, E5> x25(@Nullable E2 e2, @Nullable E5 e5)  {
    return new NTuple5<>(elem1, e2, elem3, elem4, e5);
  }

  /**
   * Create a 5-tuple from this one where the third and fifth elements are exchanged. 
   * @param e3 new third element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E3> type of exchanged third element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E3, E5> NTuple5<T1, T2, E3, T4, E5> x35(@Nullable E3 e3, @Nullable E5 e5)  {
    return new NTuple5<>(elem1, elem2, e3, elem4, e5);
  }

  /**
   * Create a 5-tuple from this one where the fourth and fifth elements are exchanged. 
   * @param e4 new fourth element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E4> type of exchanged fourth element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E4, E5> NTuple5<T1, T2, T3, E4, E5> x45(@Nullable E4 e4, @Nullable E5 e5)  {
    return new NTuple5<>(elem1, elem2, elem3, e4, e5);
  }

  /**
   * Create a 5-tuple from this one where the first, second, and thirdelements are exchanged. 
   * @param e1 new first element
   * @param e2 new second element
   * @param e3 new third element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E2> type of exchanged second element
   * @param <E3> type of exchanged third element
   */
  @NotNull
  public <E1, E2, E3> NTuple5<E1, E2, E3, T4, T5> x123(@Nullable E1 e1, @Nullable E2 e2, @Nullable E3 e3)  {
    return new NTuple5<>(e1, e2, e3, elem4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the first, second, and fourthelements are exchanged. 
   * @param e1 new first element
   * @param e2 new second element
   * @param e4 new fourth element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E2> type of exchanged second element
   * @param <E4> type of exchanged fourth element
   */
  @NotNull
  public <E1, E2, E4> NTuple5<E1, E2, T3, E4, T5> x124(@Nullable E1 e1, @Nullable E2 e2, @Nullable E4 e4)  {
    return new NTuple5<>(e1, e2, elem3, e4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the first, third, and fourthelements are exchanged. 
   * @param e1 new first element
   * @param e3 new third element
   * @param e4 new fourth element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E3> type of exchanged third element
   * @param <E4> type of exchanged fourth element
   */
  @NotNull
  public <E1, E3, E4> NTuple5<E1, T2, E3, E4, T5> x134(@Nullable E1 e1, @Nullable E3 e3, @Nullable E4 e4)  {
    return new NTuple5<>(e1, elem2, e3, e4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the second, third, and fourthelements are exchanged. 
   * @param e2 new second element
   * @param e3 new third element
   * @param e4 new fourth element
   * @return tuple with exchanged elements
   * @param <E2> type of exchanged second element
   * @param <E3> type of exchanged third element
   * @param <E4> type of exchanged fourth element
   */
  @NotNull
  public <E2, E3, E4> NTuple5<T1, E2, E3, E4, T5> x234(@Nullable E2 e2, @Nullable E3 e3, @Nullable E4 e4)  {
    return new NTuple5<>(elem1, e2, e3, e4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the first, second, and fifthelements are exchanged. 
   * @param e1 new first element
   * @param e2 new second element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E2> type of exchanged second element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E1, E2, E5> NTuple5<E1, E2, T3, T4, E5> x125(@Nullable E1 e1, @Nullable E2 e2, @Nullable E5 e5)  {
    return new NTuple5<>(e1, e2, elem3, elem4, e5);
  }

  /**
   * Create a 5-tuple from this one where the first, third, and fifthelements are exchanged. 
   * @param e1 new first element
   * @param e3 new third element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E3> type of exchanged third element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E1, E3, E5> NTuple5<E1, T2, E3, T4, E5> x135(@Nullable E1 e1, @Nullable E3 e3, @Nullable E5 e5)  {
    return new NTuple5<>(e1, elem2, e3, elem4, e5);
  }

  /**
   * Create a 5-tuple from this one where the second, third, and fifthelements are exchanged. 
   * @param e2 new second element
   * @param e3 new third element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E2> type of exchanged second element
   * @param <E3> type of exchanged third element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E2, E3, E5> NTuple5<T1, E2, E3, T4, E5> x235(@Nullable E2 e2, @Nullable E3 e3, @Nullable E5 e5)  {
    return new NTuple5<>(elem1, e2, e3, elem4, e5);
  }

  /**
   * Create a 5-tuple from this one where the first, fourth, and fifthelements are exchanged. 
   * @param e1 new first element
   * @param e4 new fourth element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E4> type of exchanged fourth element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E1, E4, E5> NTuple5<E1, T2, T3, E4, E5> x145(@Nullable E1 e1, @Nullable E4 e4, @Nullable E5 e5)  {
    return new NTuple5<>(e1, elem2, elem3, e4, e5);
  }

  /**
   * Create a 5-tuple from this one where the second, fourth, and fifthelements are exchanged. 
   * @param e2 new second element
   * @param e4 new fourth element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E2> type of exchanged second element
   * @param <E4> type of exchanged fourth element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E2, E4, E5> NTuple5<T1, E2, T3, E4, E5> x245(@Nullable E2 e2, @Nullable E4 e4, @Nullable E5 e5)  {
    return new NTuple5<>(elem1, e2, elem3, e4, e5);
  }

  /**
   * Create a 5-tuple from this one where the third, fourth, and fifthelements are exchanged. 
   * @param e3 new third element
   * @param e4 new fourth element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E3> type of exchanged third element
   * @param <E4> type of exchanged fourth element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E3, E4, E5> NTuple5<T1, T2, E3, E4, E5> x345(@Nullable E3 e3, @Nullable E4 e4, @Nullable E5 e5)  {
    return new NTuple5<>(elem1, elem2, e3, e4, e5);
  }

  /**
   * Create a 5-tuple from this one where the first, second, third, and fourthelements are exchanged. 
   * @param e1 new first element
   * @param e2 new second element
   * @param e3 new third element
   * @param e4 new fourth element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E2> type of exchanged second element
   * @param <E3> type of exchanged third element
   * @param <E4> type of exchanged fourth element
   */
  @NotNull
  public <E1, E2, E3, E4> NTuple5<E1, E2, E3, E4, T5> x1234(@Nullable E1 e1, @Nullable E2 e2, @Nullable E3 e3, @Nullable E4 e4)  {
    return new NTuple5<>(e1, e2, e3, e4, elem5);
  }

  /**
   * Create a 5-tuple from this one where the first, second, third, and fifthelements are exchanged. 
   * @param e1 new first element
   * @param e2 new second element
   * @param e3 new third element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E2> type of exchanged second element
   * @param <E3> type of exchanged third element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E1, E2, E3, E5> NTuple5<E1, E2, E3, T4, E5> x1235(@Nullable E1 e1, @Nullable E2 e2, @Nullable E3 e3, @Nullable E5 e5)  {
    return new NTuple5<>(e1, e2, e3, elem4, e5);
  }

  /**
   * Create a 5-tuple from this one where the first, second, fourth, and fifthelements are exchanged. 
   * @param e1 new first element
   * @param e2 new second element
   * @param e4 new fourth element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E2> type of exchanged second element
   * @param <E4> type of exchanged fourth element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E1, E2, E4, E5> NTuple5<E1, E2, T3, E4, E5> x1245(@Nullable E1 e1, @Nullable E2 e2, @Nullable E4 e4, @Nullable E5 e5)  {
    return new NTuple5<>(e1, e2, elem3, e4, e5);
  }

  /**
   * Create a 5-tuple from this one where the first, third, fourth, and fifthelements are exchanged. 
   * @param e1 new first element
   * @param e3 new third element
   * @param e4 new fourth element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E1> type of exchanged first element
   * @param <E3> type of exchanged third element
   * @param <E4> type of exchanged fourth element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E1, E3, E4, E5> NTuple5<E1, T2, E3, E4, E5> x1345(@Nullable E1 e1, @Nullable E3 e3, @Nullable E4 e4, @Nullable E5 e5)  {
    return new NTuple5<>(e1, elem2, e3, e4, e5);
  }

  /**
   * Create a 5-tuple from this one where the second, third, fourth, and fifthelements are exchanged. 
   * @param e2 new second element
   * @param e3 new third element
   * @param e4 new fourth element
   * @param e5 new fifth element
   * @return tuple with exchanged elements
   * @param <E2> type of exchanged second element
   * @param <E3> type of exchanged third element
   * @param <E4> type of exchanged fourth element
   * @param <E5> type of exchanged fifth element
   */
  @NotNull
  public <E2, E3, E4, E5> NTuple5<T1, E2, E3, E4, E5> x2345(@Nullable E2 e2, @Nullable E3 e3, @Nullable E4 e4, @Nullable E5 e5)  {
    return new NTuple5<>(elem1, e2, e3, e4, e5);
  }

  /**
   * Create a 6-tuple from this 5-tuple by inserting a new first element.
   * @param e1 new element
   * @param <T> new element type
   * @return 6-tuple
   */
  @NotNull
  public <T> NTuple6<T, T1, T2, T3, T4, T5> i1(@Nullable T e1)
  {
    return new NTuple6<>(e1, elem1, elem2, elem3, elem4, elem5);
  }

  /**
   * Create a 6-tuple from this 5-tuple by inserting a new second element.
   * @param e2 new element
   * @param <T> new element type
   * @return 6-tuple
   */
  @NotNull
  public <T> NTuple6<T1, T, T2, T3, T4, T5> i2(@Nullable T e2)
  {
    return new NTuple6<>(elem1, e2, elem2, elem3, elem4, elem5);
  }

  /**
   * Create a 6-tuple from this 5-tuple by inserting a new third element.
   * @param e3 new element
   * @param <T> new element type
   * @return 6-tuple
   */
  @NotNull
  public <T> NTuple6<T1, T2, T, T3, T4, T5> i3(@Nullable T e3)
  {
    return new NTuple6<>(elem1, elem2, e3, elem3, elem4, elem5);
  }

  /**
   * Create a 6-tuple from this 5-tuple by inserting a new fourth element.
   * @param e4 new element
   * @param <T> new element type
   * @return 6-tuple
   */
  @NotNull
  public <T> NTuple6<T1, T2, T3, T, T4, T5> i4(@Nullable T e4)
  {
    return new NTuple6<>(elem1, elem2, elem3, e4, elem4, elem5);
  }

  /**
   * Create a 6-tuple from this 5-tuple by inserting a new fifth element.
   * @param e5 new element
   * @param <T> new element type
   * @return 6-tuple
   */
  @NotNull
  public <T> NTuple6<T1, T2, T3, T4, T, T5> i5(@Nullable T e5)
  {
    return new NTuple6<>(elem1, elem2, elem3, elem4, e5, elem5);
  }

  /**
   * Create a 6-tuple from this 5-tuple by adding a new sixth element.
   * @param e6 new element
   * @param <T> new element type
   * @return 6-tuple
   */
  @NotNull
  public <T> NTuple6<T1, T2, T3, T4, T5, T> i6(@Nullable T e6)
  {
    return new NTuple6<>(elem1, elem2, elem3, elem4, elem5, e6);
  }

  @NotNull
  @Override
  public String toString()
  {
    return ITuple5.toString("NTuple5", this);
  }

}
